%* crtdds_study                                                                   *;
%*                                                                                *;
%* Creates the SAS CRT-DDS study data set from source metadata.                   *;
%*                                                                                *;
%* @param _cstname - required - The /ODM/Study/GlobalVariables/StudyName          *;
%*            attribute.                                                          *;
%* @param _cstdescr - required - The /ODM/Study/GlobalVariables/StudyDescription  *;
%*            attribute.                                                          *;
%* @param _cstprotocol - required - The /ODM/Study/GlobalVariables/ProtocolName   *;
%*            attribute.                                                          *;
%* @param _cstdefineds - required - The DefineDocument data set in the output     *;
%*            library (for example, srcdata.DefineDocument).                      *;
%* @param _cstdefinename - required - The value of the ID column in the           *;
%*            _cstdefineds data set.                                              *;
%* @param _cstoutstudyds - required - The Study data set to create                *;
%*            (for example, srcdata.Study).                                       *;
%*                                                                                *;
%* @since 1.2                                                                     *;
%* @exposure external                                                             *;

%macro crtdds_study(
    _cstname=,
    _cstdescr=,
    _cstprotocol=,
    _cstdefineds=,
    _cstdefinename=,
    _cstoutstudyds=
    ) / des ="Creates SAS CRT-DDS Study data set";

%local _cstRandom ds keycnt key cnt add src2 defds rc dsidst;

%if %length(&_cstname)=0 or %length(&_cstdescr)=0 or %length(&_cstprotocol)=0
  or %length(&_cstdefineds)=0 or %length(&_cstdefinename)=0 or %length(&_cstoutstudyds)=0
    %then %goto exit;

%if ^%sysfunc(exist(&_cstdefineDS)) %then %goto exit;

 %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let ds=_s&_cstRandom;
  %let keycnt=1;
  %let cnt=0;
  %let add=1;

%* need to check if 2 level name passed in for parameters, pull off dataset name for use in proc sql *;
  %if %symexist(_cstdefineDS) %then %do;
      %if (%sysfunc(indexc("&_cstdefineDS",'.'))=0 )%then %let defds=&_cstdefineDS;
      %else %do;
        %let src2=%scan(&_cstdefineDS,2,'.');
    %let defds=&src2;
    %end;
  %end;

 %if %symexist(_cstoutstudyds) %then %do;
    %if %sysfunc(exist(&_cstoutStudyDS)) %then %do;
      %let dsidst=%sysfunc(open(&_cstoutStudyDS,i));
      %if &dsidst>0 %then %do;
        %let key=%sysfunc(attrn(&dsidst, nlobs));
        %let rc=%sysfunc(close(&dsidst));
        %if &key>0 %then %do;
          proc sql noprint;
          select count(distinct oid) into :cnt
          from &_cstoutstudyds
          where upcase(studyName)=upcase("&_cstName");
          quit;
        %end;
       %end;
      data _null_;
      keycnt=strip(sum(&key,1));
      call symput('keycnt',keycnt);
      run;
  %if &cnt>0 %then %let add=0;
    %end;
  %end;

%if &add=1 %then %do;
  proc sql;
  create table &ds as
  select
      strip("&keycnt") as OID length=128,
      "&_cstname" as StudyName length=128,
      "&_cstdescr" as StudyDescription length=2000,
      "&_cstprotocol" as ProtocolName length=128,
      &defds..fileoid as FK_DefineDocument
    from &_cstdefineDS
    where ( upcase(id) = upcase("&_cstdefinename"));
    quit;
proc append base=&_cstoutstudyds data=&ds force;
run;
%end;
%else %put Study &_cstname already exists in this file;


%if %sysfunc(exist(&ds)) %then %do;
  proc datasets nolist lib=work;
  delete &ds;
  quit;
  run;
%end;
%exit:

%mend crtdds_study;
