%* ct_xmlvalidate                                                                 *;
%*                                                                                *;
%* Validates an XML file against the CT V1.0.0 XML W3C schema.                    *;
%*                                                                                *;
%* General use of this macro is in combination with another macro (such as        *;
%* ct_write or ct_read). If this macro is run independently, conditional code     *;
%* is included that optionally writes metadata to the Results data set and        *;
%* optionally checks the validity of the SASReferences data set.                  *;
%*                                                                                *;
%* @macvar _cstDebug Turns debugging on or off for the session                    *;
%* @macvar _cst_rc Task error status                                              *;
%* @macvar _cst_rcmsg Message associated with _cst_rc                             *;
%* @macvar _cstGRoot Root path of the Toolkit Global Library                      *;
%* @macvar studyRootPath Root path of the Study Library                           *;
%* @macvar _cstSASRefs Run-time SASReferences data set derived in process setup   *;
%* @macvar _cstResultFlag Results: Problem detected?                              *;
%* @macvar _cstResultsDS Results data set                                         *;
%* @macvar _cstResultSeq Results: Unique invocation of check                      *;
%* @macvar _cstSeqCnt Results: Sequence number within _cstResultSeq               *;
%*                                                                                *;
%* @param _cstLogLevel - required - The level of error reporting.                 *;
%*            Values: info|warning|error|fatal error                              *;
%*            Default: info                                                       *;
%* @param _cstResultsOverrideDS - optional - The (libname.)member that refers to  *;
%*            a Results data set to create. If omitted, the Results data set that *;
%*            is specified by &_cstResultsDS is used.                             *;
%*                                                                                *;
%* @since 1.5                                                                     *;
%* @exposure external                                                             *;

%macro ct_xmlvalidate(
    _cstLogLevel=info,
    _cstResultsOverrideDS=
    ) / des='CST: CT V1.0.0 XML schema validation';

  %* declare local variables used in the macro;
  %local
    _cstRandom
    _cstGlobalMDPath
    _cstGlobalXSLPath
    _cstGlobalXSDPath
    _cstGlobalTransformsXML
    _cstGlobalStdDS
    _cstGlobalStdSASRefsDS
    _cstGlobalMDLib

    _cstThisMacroRC
    _cstTempDS1

    _cstError
    _cstParm1
    _cstParm2
    _cstNextCode

    _cstTypeRefXML
    _cstTypeExtXML
    _cstTypeSourceData
    _cstTypeRefMD
    _cstSubTypeTable
    _cstSubtypeXML
    _cstSubtypeStylesheet
    _cstSubtypeXsdRepos
    _cstSubtypeXslRepos

    _cstStd
    _cstStdVsn
    _cstAction

    _cstXsdReposPath
    _cstXslReposPath
    _cstParamsClass
    _cstTransformsClass
    _cstJavaPicklist
    _cstWorkPath
    _cstAvailableTransformsPath
    _cstExternalXMLPath
    _cstStyleSheetPath
    _cstOutputStyleSheetName
    _cstCubeXMLName
    _cstCubeXMLPath
    _cstLogXMLName
    _cstLogXMLPath

    _cstXMLEngine

    _cstSrcDataLibrary
    _cstResultsDSLib

    _cstTempFilename1
    _cstTempFilename2
    _cstTempFilename3
    _cstTempFilename4
    _cstTempFilename5
    _cstTempDS1
    _cstTempDirLib1
    _cstTempDirLib1FullPath
    _cstNextCode

    _cstTempRefMdTable

    _cstSavedOrigResultsName
    _cstLineNumberColExists

    _cstNewProcess
    _cstrundt
    _cstrunsasref
    _cstrunstd
    _cstrunstdver
    _cstVer
    _cstSrcData
    ;

  %let _cstSrcData=&sysmacroname;

  %* Make copy of global macro variables;
  %let _cstStd=&_cstStandard;
  %let _cstStdVsn=&_cstStandardVersion;

  %let _cstThisMacroRC=0;
  %if %length(&_cstResultsOverrideDS)>0 %then
    %let _cstresultsds=&_cstResultsOverrideds;

  data _null_;
   if upcase("&_cstLogLevel") in ("INFO", "ERROR" , "WARNING", "FATAL ERROR")
      then call symput('loglev',"&_cstLogLevel");
   else call symput('loglev',"INFO");
  run;


  %* Generate the random names used in the macro;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstGlobalMDLib=_gmd&_cstRandom;
  %let _cstTempDS1=_t1&_cstRandom;
  %let _cstTempFilename2=_t2&_cstRandom;
  %let _cstTempFilename3=_t3&_cstRandom;
  %let _cstTempFilename4=_t4t&_cstRandom;
  %let _cstLogXMLName=_log&_cstRandom;
  %let _cstTempRefMdTable=_tmd&_cstRandom;
  %let _cstNextCode=_cod&_cstRandom;
  %let _cstTempDS1=_ds1&_cstRandom;
  %let _cstTempDirLib1=_tl&_cstRandom;

  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDirLib1=_cst&_cstRandom;

  %* Determine XML engine;
  %let _cstXMLEngine=xml;
  %if %eval(&SYSVER EQ 9.2) %then %let _cstXMLEngine=xml92;
  %if %eval(&SYSVER GE 9.3) %then %let _cstXMLEngine=xmlv2;

  %* Create a temporary messages data set if required;
  %local _cstNeedToDeleteMsgs;
  %cstutil_createTempMessages(_cstCreationFlag=_cstNeedToDeleteMsgs);

  %* Set up the location of the results data set to write to;
  %local
    _cstThisResultsDS
    _cstThisResultsDSLib
    _cstThisResultsDSMem
    _cstUsingResultsOverride
    _cstSaveResultSeq
    _cstSaveSeqCnt
    ;
  %cstutil_internalManageResults(_cstAction=SAVE);
  %let _cstResultSeq=1;
  %let _cstSeqCnt=0;

  * assign a filename to the catalog that will hold the next code to run;
  data _null_;
  call symputx('_cstWorkPath',pathname('work'),'L');
  run;
 filename &_cstNextCode  "&_cstWorkPath/&_cstNextCode..sas";

  * The sasrefs data set must exist;
  %if (^(%sysfunc(exist(&_cstsasrefs)))) %then %do;
     %let _cstParam1=SASReferences;
     %let _cstParam2=&_cstsasrefs;
     %goto MISSING_DATASET;
  %end;

  %* Check to see if the current results data set contains metadata about the current process;
  %* If so, this is a continuation of the process and the following metadata need not be     ;
  %*  written to the results data set.                                                       ;
  %let _cstNewProcess=1;
  %if (%sysfunc(exist(&_cstResultsDS))) %then
  %do;
    data _null_;
      set &_cstResultsDS (where=(checkid='' and substr(message,1,7)="PROCESS")) end=last;
      if message =: "PROCESS STANDARD: CT" then
        call symputx('_cstNewProcess',0);
    run;
  %end;

  data _null_;
    set &_cstSASrefs (where=(upcase(standard)="CDISC-CT"));

    attrib _csttemp format=$500. label='Temporary variable string';

    if _n_=1 then do;
      call symputx('_cstrundt',put(datetime(),is8601dt.));
      call symputx('_cstrunstd',standard);
      call symputx('_cstrunstdver',standardversion);
    end;

    if upcase(type)="CONTROL" and upcase(subtype)="REFERENCE" then
    do;
      if path ne '' and memname ne '' then
      do;
        if kindexc(ksubstr(kreverse(path),1,1),'/\') then
          _csttemp=catx('',path,memname);
       else
          _csttemp=catx('/',path,memname);
      end;
      else
        _csttemp="&_cstsasrefs";
      call symputx('_cstrunsasref',_csttemp);
    end;

  run;

  %if %length(&_cstrunsasref)=0 %then
  %do;
    %if %sysfunc(indexc(&_cstsasrefs,'.')) %then
    %do;
      %let _cstTempLib=%SYSFUNC(scan(&_cstsasrefs,1,'.'));
      %let _cstTempDS=%SYSFUNC(scan(&_cstsasrefs,2,'.'));
    %end;
    %else
    %do;
      %let _cstTempLib=work;
      %let _cstTempDS=&_cstsasrefs;
    %end;
    %let _cstrunsasref=%sysfunc(pathname(&_cstTempLib))/&_cstTempDS..sas7bdat;
  %end;

  %cst_getRegisteredStandards(_cstOutputDS=work._cstStandards);
  data _null_;
    set work._cstStandards (where=(standard="CST-FRAMEWORK"));
    call symputx('_cstVer',strip(productrevision));
  run;
  %cstutil_deleteDataSet(_cstDataSetName=work._cstStandards);

  %if (&_cstNewProcess) %then
  %do;
    %*************************************************************;
    %* Write information to the results data set about this run. *;
    %*************************************************************;
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARD: &_cstrunstd,_cstSeqNoParm=1,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARDVERSION: &_cstrunstdver,_cstSeqNoParm=2,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DRIVER: CREATE_CTXML,_cstSeqNoParm=3,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DATE: &_cstrundt,_cstSeqNoParm=4,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS TYPE: XMLVALIDATE CT ,_cstSeqNoParm=5,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS SASREFERENCES: &_cstrunsasref,_cstSeqNoParm=6,_cstSrcDataParm=&_cstSrcData);
    %if %symexist(studyRootPath) %then
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: &studyRootPath,_cstSeqNoParm=7,_cstSrcDataParm=&_cstSrcData);
    %else
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: <not used>,_cstSeqNoParm=7,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS GLOBALLIBRARY: &_cstGRoot,_cstSeqNoParm=8,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS CSTVERSION: &_cstVer,_cstSeqNoParm=9,_cstSrcDataParm=&_cstSrcData);
    %let _cstSeqCnt=9;

    %* check that the sas references table is correct;
    %* need to redirect the results because there is no override in checkDS;
    %let _cstSavedOrigResultsName=&_cstResultsDS;
    %let _cstResultsDS=&_cstThisResultsDS;

    %if (&_cstDebug) %then %do;
      %put before the call to cst_checkDS;
      %put _cstSavedOrigResultsName=&_cstSavedOrigResultsName;
      %put _cstResultsDS=&_cstResultsDS;
    %end;

  %end;
  %else
  %do;
    %let _cstSeqCnt=1;
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Starting XML Validation,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
  %end;

  * The library for the results must exist;
  %if ((%sysfunc(libref(&_cstThisResultsDSLib)))) %then %do;
     %let _cstParam1=&_cstThisResultsDSLib;
     %goto RESULTSLIB_NOT_ASSIGNED;
  %end;

  %* set up static-final style variables;
  %let _cstAction=EXPORT;


  %* Get the fileref of the XML file;
  %cstUtil_getSASReference(
    _cstStandard=%upcase(&_cstrunstd)
    ,_cstStandardVersion=&_cstrunstdver
    ,_cstSASRefType=externalxml
    ,_cstSASRefSubType=xml
    ,_cstSASRefsasref=_cstTempFilename1
    );
  %if (&_cst_rc) %then %do;
     %let _cstParam1=&_cstTypeExtXml/&_cstSubtypeXML;
     %goto MISSING_SASREF;
  %end;
  %else %if (&_cstDebug=1) %then %do;
    %put XML file to create from sasrefs file: &_cstTempFilename1;
  %end;


  * Save the path to the XML file, xsd/xsl repos and work library for use later;
  data _null_;
    xmlpath=ktranslate(pathname("&_cstTempFilename1"),'/','\');
    call symputx('_cstExternalXMLPath',xmlpath);
  run;

  %let _cstXsdReposPath=&_cstGRoot/schema-repository;
  %let _cstParamsClass=com/sas/ptc/transform/xml/StandardXMLTransformerParams;
  %let _cstTransformsClass=com/sas/ptc/transform/xml/StandardXMLExporter;
  %let _cstJavaPicklist=cstframework/cstframework.txt;
  %let _cstLogXMLPath=&_cstWorkPath./&_cstLogXMLName;
  %let _cstAvailableTransformsPath=&_cstgroot/metadata/availabletransforms.xml;

  %* Print out debugging information;
  %if (&_cstDebug=1) %then %do;
    %put Note: Calling java with parameters:;
    %put       _cstJavaPicklist=&_cstJavaPicklist;
    %put       _cstParamsClass=&_cstParamsClass;
    %put       _cstTransformsClass=&_cstTransformsClass;
    %put       _cstAction=&_cstAction;
    %put       _cstStandard=&_cstrunstd;
    %put       _cstStandardVersion=&_cstrunstdver ;
    %put       _cstXsdReposPath=&_cstXsdReposPath;
    %put       _cstExternalXMLPath=&_cstExternalXMLPath;
    %put       _cstAvailableTransformsPath=&_cstAvailableTransformsPath;
    %put       _cstLogXMLPath=&_cstLogXMLPath;
  %end;


  %* In the following, the logging level is set to info as the transform creates
     an empty XML doc if nothing is reported, which causes an error in the SAS libname;
  * Create the external XML file from intermediate xml;
  data _null_
    %* Only use picklist for SAS V9.2 and above;
    %if %eval(^ (&SYSVER=9.1)) %then %do;
         / picklist="&_cstJavaPicklist"
    %end;
    ;

    dcl javaobj prefs("&_cstParamsClass");
    prefs.callvoidmethod('setImportOrExport',"&_cstAction");
    prefs.callvoidmethod('setStandardName',"&_cstrunstd");
    prefs.callvoidmethod('setStandardVersion',"&_cstrunstdver");
    prefs.callvoidmethod('setValidatingStandardXMLString', "true");
    prefs.callvoidmethod('setValidatingXMLOnlyString', "true");
    prefs.callvoidmethod('setStandardXMLPath', "&_cstExternalXMLPath");
    prefs.callvoidmethod('setSchemaBasePath',"&_cstXsdReposPath");
    prefs.callvoidmethod('setAvailableTransformsFilePath',"&_cstAvailableTransformsPath");
    prefs.callvoidmethod('setLogFilePath',"&_cstLogXMLPath");

    prefs.callvoidmethod('setLogLevelString',"&_cstLogLevel");

    dcl javaobj transformer("&_cstTransformsClass", prefs);
    transformer.exceptiondescribe(1);
    transformer.callvoidmethod('exec');

    * check the return values here and get results path;
    transformer.delete();
    prefs.delete();
  run;

  * check to see if there are any Java issues;
  %cstutilcheckjava;
  %if (&_cstResultFlag) %then %do;
    %goto JAVA_ERRORS;
  %end;

  %if %sysfunc(fileexist("&_cstLogXMLPath")) eq 1 %then
  %do;

    %LET NORECS=1;
    data _null_;
      infile "&_cstlogXMLPath" truncover ;
        input @ ;
        input x $8.;
        if _n_=2 and x='<TABLE/>' THEN CALL SYMPUT('NORECS','0');
    run;

    %if "&norecs"="1" %then
    %do;

      %* reuse the same temp variable name for file name;
      * Assign a libname to the log XML file;
      libname &_cstLogXMLName &_cstXMLEngine "&_cstLogXMLPath";

      * Check to see if the line number/column number info was generated;
      proc contents noprint
        data=&_cstLogXMLName..XMLTransformLog out=&_cstTempDS1;
      run;

      %let _cstLineNumberColExists=0;
      data _null_;
        set &_cstTempDS1;
        if (upcase(name)="LINENUMBER") then
          call symputx('_cstLineNumberColExists',1);
      run;

      %if (&_cstDebug) %then
      %do;
        %put _cstLineNumberColExists=&_cstLineNumberColExists;

        %put Results from the Java call:;
        data _null_;
          set &_cstLogXMLName..XMLTransformLog;
          put message=;
        run;
      %end;

      * Create a work results data set to capture the XML log information;
      data work._cstxmllog;
        %cstutil_resultsdskeep;
        %cstutil_resultsdsattr;
        length _cstmsgParm1 $450 _cstmsgParm2 $1;
        keep _cstmsgParm1 _cstmsgParm2;
        call missing(actual,keyvalues,resultdetails,_cstmsgParm2);

        set &_cstLogXMLName..XMLTransformLog;

        seqno=_n_;
        resultseq=1;
        checksource='XMLTRANSFORM';

        %if (&_cstLineNumberColExists) %then
        %do;
          if (lineNumber^=.) then do;
            _cstMsgParm1='(Line ' || compress(put(lineNumber,8.)) ||
               '/Column ' || compress(put(columnNumber,8.)) || ') ' || trim(left(message));
            message='(Line ' || compress(put(lineNumber,8.)) ||
               '/Column ' || compress(put(columnNumber,8.)) || ') ' || trim(left(message));
          end;
          else do;
            _cstmsgparm1=strip(message);
            message=strip(message);
          end;
        %end;
        %else
        %do;
          _cstmsgparm1=strip(message);
          message=strip(message);
        %end;

        resultseverity=severity;
        srcdata=Strip(origin);
        checkId='';
        ResultFlag=0;

        if (severity='INFO') then
          ResultId='CT0001';
        else if (severity='WARNING') then
          ResultId='CT0002';
        else do;
          * ERROR/CRITICAL ERROR;
          ResultId='CT0003';
          ResultFlag=1;
          call symputx('_cstThisMacroRC','1','L');
        end;
        _cst_RC=ResultFlag;
      run;

      %cstutil_appendresultds(
         _cstErrorDS=work._cstxmllog,
         _cstSource=XMLTRANSFORM,
         _cstVersion=1.0,
         _cstOrderBy=seqno);

      * Clear the libname;
      libname &_cstLogXMLName;

      %cstutil_deleteDataSet(_cstDataSetName=work._cstxmllog);

    %end;
  %end;

  %* Handle any errors generated during the java code;
  %if (&_cstThisMacroRC) %then %do;
    %goto GENERATION_ERRORS;
  %end;

  %* Everything was OK so  report it;
  %let _cstParam1=&_cstExternalXMLPath;
  %let _cstThisMacroRC=0;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0115
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );

  %goto CLEANUP;


%RESULTSLIB_NOT_ASSIGNED:
  %if (&_cstDebug) %then %do;
     %put In RESULTSLIB_NOT_ASSIGNED;
  %end;

  %local _cstTempResultsDS;

  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempResultsDS=_cst&_cstRandom;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0007
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstTempResultsDS
                );

  %* print the temp results to the log as the resutls file cannot exist;
  data _null_;
    file &_cstNextCode;
    set &_cstTempResultsDS;
    resultSeverity=upcase(resultSeverity);
    put %nrstr("%put ") resultSeverity ": "  message ";";
  run;

  %cstutil_deleteDataSet(_cstDataSetName=work.&_cstTempResultsDS);

  %include &_cstNextCode;

  %let rc=delete("&_cstNextCode");

  %goto CLEANUP;

%MISSING_DATASET:
  %if (&_cstDebug) %then %do;
     %put In MISSING_DATASET;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0008
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_FILE:
  %if (&_cstDebug) %then %do;
     %put In MISSING_FILE;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0009
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_SASREF:
  %if (&_cstDebug) %then %do;
     %put In MISSING_SASREF;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0004
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_ASSIGNMENT:
  %if (&_cstDebug) %then %do;
     %put In MISSING_ASSIGNMENT;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0005
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%INVALID_PARAM_VALUE:
  %if (&_cstDebug) %then %do;
     %put In INVALID_PARAM_VALUE;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0006
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%GENERATION_ERRORS:
  %if (&_cstDebug) %then %do;
     %put In GENERATION_ERRORS;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0011
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%JAVA_ERRORS:
  %if (&_cstDebug) %then %do;
     %put In JAVA_ERRORS;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=CT0099
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=&_cstSrcData
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                ,_cstResultParm1=PROCESS HALTED
                );
  %goto CLEANUP;

%CLEANUP:
  %if (&_cstDebug) %then %do;
     %put In CLEANUP;
  %end;

  %* Reset global macro variables;
  %let _cstStandard=&_cstStd;
  %let _cstStandardVersion=&_cstStdVsn;

  %* Persist the results if specified in sasreferences  *;
  %cstutil_saveresults();

  %* reset the resultSequence/SeqCnt variables;
  %cstutil_internalManageResults(_cstAction=RESTORE);

  * Clear the libname;

  * Clear the temporary filename into the work catalog;
  filename &_cstNextCode;

  * Clean up temporary data sets if they exist;
  %cstutil_deleteDataSet(_cstDataSetName=work.&_cstTempDS1);


  %* Delete the temporary messages data set if it was created here;
  %if (&_cstNeedToDeleteMsgs=1) %then %do;
    %local _cstMsgDir _cstMsgMem;
    %if %eval(%index(&_cstMessages,.)>0) %then %do;
      %let _cstMsgDir=%scan(&_cstMessages,1,.);
      %let _cstMsgMem=%scan(&_cstMessages,2,.);
    %end;
    %else %do;
      %let _cstMsgDir=work;
      %let _cstMsgMem=&_cstMessages;
    %end;
    %cstutil_deleteDataSet(_cstDataSetName=&_cstMsgDir..&_cstMsgMem);
  %end;

  %let _cst_rc=&_cstThisMacroRC;

%mend ct_xmlvalidate;
