%* odm_write                                                                      *;
%*                                                                                *;
%* Writes an ODM 1.3.0 XML file.                                                  *;
%*                                                                                *;
%* This macro uses the SAS representation of an ODM XML file as source data and   *;
%* converts it to the required XML structure.  The inputs and outputs are         *;
%* specified via in a  SASReferences file.                                        *;
%*                                                                                *;
%* @macvar _cstDebug Turns debugging on or off for the session                    *;
%* @macvar _cst_rc Task error status                                              *;
%* @macvar _cst_rcmsg Message associated with _cst_rc                             *;
%* @macvar _cstSASRefs Run-time SASReferences data set derived in process setup   *;
%* @macvar _cstResultFlag Results: Problem detected?                              *;
%* @macvar _cstResultsDS Results data set                                         *;
%* @macvar _cstResultSeq Results: Unique invocation of check                      *;
%* @macvar _cstSeqCnt Results: Sequence number within _cstResultSeq               *;
%*                                                                                *;
%* @param _cstCreateDisplayStyleSheet - optional - Create a style sheet in the    *;
%*            same directory as the output XML file.                              *;
%*            1: This macro looks in SASReferences for a record with a type or    *;
%*            subtype of referencexml/stylesheet and uses that record.            *;
%*            0: This macro does not create the XSL, even if a record is          *;
%*            specified in SASReferences.                                         *;
%*            Values: 0|1                                                         *;
%*            Default: 0                                                          *;
%* @param _cstOutputEncoding - optional - The XML encoding to use for the ODM     *;
%*            file that is created.                                               *;
%*            Default: UTF-8                                                      *;
%* @param _cstHeaderComment - optional - A short comment that is added to the top *;
%*            of the ODM XML file that is produced. If no comment is provided, a  *;
%*            default is used.                                                    *;
%* @param _cstResultsOverrideDS - optional - The (libname.)member that refers to  *;
%*            a Results data set to create. If omitted, the Results data set that *;
%*            is specified by &_cstResultsDS is used.                             *;
%* @param _cstLogLevel - required - The level of error reporting.                 *;
%*            Values: info|warning|error|fatal error                              *;
%*            Default: info                                                       *;
%*                                                                                *;
%* @since 1.4                                                                     *;
%* @exposure external                                                             *;

%macro odm_write(
    _cstCreateDisplayStyleSheet=0,
    _cstOutputEncoding=UTF-8,
    _cstHeaderComment=,
    _cstResultsOverrideDS=,
    _cstLogLevel=info
    ) / des='CST: Write ODM 1.3.0 XML file';


  %* declare local variables used in the macro;
  %local
    _cstRandom
    _cstGlobalMDPath
    _cstGlobalXSLPath
    _cstGlobalXSDPath
    _cstGlobalTransformsXML
    _cstGlobalStdDS
    _cstGlobalStdSASRefsDS
    _cstGlobalMDLib

    _cstThisMacroRC
    _cstTempDS1

    _cstError
    _cstParm1
    _cstParm2
    _cstNextCode

    _cstTypeRefXML
    _cstTypeExtXML
    _cstTypeSourceData
    _cstTypeRefMD
    _cstSubTypeTable
    _cstSubtypeXML
    _cstSubtypeStylesheet
    _cstSubtypeXsdRepos
    _cstSubtypeXslRepos

    _cstStandard
    _cstStandardVersion
    _cstAction

    _cstXsdReposPath
    _cstXslReposPath
    _cstParamsClass
    _cstTransformsClass
    _cstJavaPicklist
    _cstWorkPath
    _cstAvailableTransformsPath
    _cstExternalXMLPath
    _cstStyleSheetPath
    _cstOutputStyleSheetName
    _cstCubeXMLName
    _cstCubeXMLFilename
    _cstCubeXMLPath
    _cstLogXMLName
    _cstLogXMLPath

    _cstXMLEngine

    _cstSrcDataLibrary
    _cstResultsDSLib

    _cstTempFilename1
    _cstTempFilename2
    _cstTempFilename3
    _cstTempFilename4
    _cstTempFilename5
    _cstTempDS1
    _cstTempDirLib1
    _cstTempDirLib1FullPath
    _cstNextCode

    _cstTempRefMdTable
    _cstTempRefMdTableObs

    _cstSavedOrigResultsName
    _cstLineNumberColExists

    _cstrundt
    _cstrunsasref
    _cstrunstd
    _cstrunstdver
    _cstVer
    ;

  %let _cstrundt=;
  %let _cstrunsasref=;
  %let _cstrunstd=;
  %let _cstrunstdver=;
  %let _cstVer=1.2;

  %let _cstThisMacroRC=0;
  %****if %symexist(_cstResultsOverrideDS) %then %let _cstresultsds=&_cstResultsOverrideds;
  %if %length(&_cstResultsOverrideDS)>0 %then
    %let _cstresultsds=&_cstResultsOverrideds;

  %* retrieve static variables;
  %cst_getStatic(_cstName=CST_GLOBALMD_PATH,_cstVar=_cstGlobalMDPath);
  %cst_getStatic(_cstName=CST_GLOBALMD_REGSTANDARD,_cstVar=_cstGlobalStdDS);
  %cst_getStatic(_cstName=CST_GLOBALMD_SASREFS,_cstVar=_cstGlobalStdSASRefsDS);
  %cst_getStatic(_cstName=CST_GLOBALXSD_PATH,_cstVar=_cstGlobalXsdPath);
  %cst_getStatic(_cstName=CST_GLOBALXSL_PATH,_cstVar=_cstGlobalXSLPath);
  %cst_getStatic(_cstName=CST_GLOBALMD_TRANSFORMSXML,_cstVar=_cstGlobalTransformsXML);

  %cst_getStatic(_cstName=CST_SASREF_TYPE_REFMD,_cstVar=_cstTypeRefMD);
  %cst_getStatic(_cstName=CST_SASREF_TYPE_SOURCEDATA,_cstVar=_cstTypeSourceData);
  %cst_getStatic(_cstName=CST_SASREF_SUBTYPE_TABLE,_cstVar=_cstSubTypeTable);

  %odm_getStatic(_cstName=ODM_SASREF_TYPE_REFXML,_cstVar=_cstTypeRefXML);
  %odm_getStatic(_cstName=ODM_SASREF_TYPE_EXTXML,_cstVar=_cstTypeExtXML);
  %odm_getStatic(_cstName=ODM_SASREF_SUBTYPE_XML,_cstVar=_cstSubtypeXML);
  %odm_getStatic(_cstName=ODM_SASREF_SUBTYPE_STYLESHEET,_cstVar=_cstSubtypeStylesheet);

  %odm_getStatic(_cstName=ODM_JAVA_PARAMSCLASS,_cstVar=_cstParamsClass);
  %odm_getStatic(_cstName=ODM_JAVA_EXPORTCLASS,_cstVar=_cstTransformsClass);
  %odm_getStatic(_cstName=ODM_JAVA_PICKLIST,_cstVar=_cstJavaPicklist);


  %* Generate the random names used in the macro;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstGlobalMDLib=_gmd&_cstRandom;
  %let _cstTempDS1=_t1&_cstRandom;
  %let _cstCubeXMLName=_cub&_cstRandom..xml;
  %let _cstCubeXMLFilename=_cub&_cstRandom;
  %let _cstTempFilename2=_t2&_cstRandom;
  %let _cstTempFilename3=_t3&_cstRandom;
  %let _cstTempFilename4=_t4t&_cstRandom;
  %let _cstLogXMLName=_log&_cstRandom;
  %let _cstTempRefMdTable=_tmd&_cstRandom;
  %let _cstNextCode=_cod&_cstRandom;
  %let _cstTempDS1=_ds1&_cstRandom;
  %let _cstTempDirLib1=_tl&_cstRandom;


  %* Determine XML engine;
  %let _cstXMLEngine=xml;
  %if %eval(&SYSVER EQ 9.2) %then %let _cstXMLEngine=xml92;
  %if %eval(&SYSVER GE 9.3) %then %let _cstXMLEngine=xmlv2;


  data _null_;
   if upcase("&_cstLogLevel") in ("INFO", "ERROR" , "WARNING", "FATAL ERROR")
      then call symput('loglev',"&_cstLogLevel");
   else call symput('loglev',"INFO");
  run;


  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDirLib1=_cst&_cstRandom;


  %* Create a temporary messages data set if required;
  %local _cstNeedToDeleteMsgs;
  %cstutil_createTempMessages(_cstCreationFlag=_cstNeedToDeleteMsgs);

  %* Set up the location of the results data set to write to;
  %local
    _cstThisResultsDS
    _cstThisResultsDSLib
    _cstThisResultsDSMem
    _cstUsingResultsOverride
    _cstSaveResultSeq
    _cstSaveSeqCnt
    ;
  %cstutil_internalManageResults(_cstAction=SAVE);
  %let _cstResultSeq=1;
  %let _cstSeqCnt=0;

  * Assign the libname to the global metadata library;
  libname &_cstGlobalMDLib "%unquote(&_cstGlobalMDPath)" access=readonly;

  * assign a filename to the catalog that will hold the next code to run;
  data _null_;
  call symputx('_cstWorkPath',pathname('work'),'L');
  run;
  filename &_cstNextCode  "&_cstWorkPath/&_cstNextCode..sas";

  * The sasrefs data set must exist;
  %if (^(%sysfunc(exist(&_cstsasrefs)))) %then %do;
     %let _cstParam1=SASReferences;
     %let _cstParam2=&_cstsasrefs;
     %goto MISSING_DATASET;
  %end;


  data _null_;
    set &_cstSASrefs (where=(upcase(standard)="CDISC-ODM"));

    attrib _csttemp format=$500. label='Temporary variable string';

    if _n_=1 then do;
      call symputx('_cstrundt',put(datetime(),is8601dt.));
      call symputx('_cstrunstd',standard);
      call symputx('_cstrunstdver',standardversion);
    end;

    if upcase(type)="CONTROL" and upcase(subtype)="REFERENCE" then
    do;
      if path ne '' and memname ne '' then
      do;
        if kindexc(ksubstr(kreverse(path),1,1),'/\') then
          _csttemp=catx('',path,memname);
        else
          _csttemp=catx('/',path,memname);
      end;
      else
        _csttemp="&_cstsasrefs";
      call symputx('_cstrunsasref',_csttemp);
    end;

  run;

  %if %length(&_cstrunsasref)=0 %then
  %do;
    %if %sysfunc(indexc(&_cstsasrefs,'.')) %then
    %do;
      %let _cstTempLib=%SYSFUNC(scan(&_cstsasrefs,1,'.'));
      %let _cstTempDS=%SYSFUNC(scan(&_cstsasrefs,2,'.'));
    %end;
    %else
    %do;
      %let _cstTempLib=work;
      %let _cstTempDS=&_cstsasrefs;
    %end;
    %let _cstrunsasref=%sysfunc(pathname(&_cstTempLib))/&_cstTempDS..sas7bdat;
  %end;

  %cst_getRegisteredStandards(_cstOutputDS=work._cstStandards);
  data _null_;
    set work._cstStandards (where=(standard="CST-FRAMEWORK"));
    call symputx('_cstVer',strip(productrevision));
  run;
  %cstutil_deleteDataSet(_cstDataSetName=work._cstStandards);


  %*************************************************************;
  %* Write information to the results data set about this run. *;
  %*************************************************************;
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARD: &_cstrunstd,_cstSeqNoParm=1,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARDVERSION: &_cstrunstdver,_cstSeqNoParm=2,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DRIVER: CREATE_ODMXML,_cstSeqNoParm=3,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DATE: &_cstrundt,_cstSeqNoParm=4,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS TYPE: CREATE ODM XML ,_cstSeqNoParm=5,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS SASREFERENCES: &_cstrunsasref,_cstSeqNoParm=6,_cstSrcDataParm=ODM_WRITE);
  %if %symexist(studyRootPath) %then
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: &studyRootPath,_cstSeqNoParm=7,_cstSrcDataParm=ODM_WRITE);
  %else
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: <not used>,_cstSeqNoParm=7,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS GLOBALLIBRARY: &_cstGRoot,_cstSeqNoParm=8,_cstSrcDataParm=ODM_WRITE);
  %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS CSTVERSION: &_cstVer,_cstSeqNoParm=9,_cstSrcDataParm=ODM_WRITE);
  %let _cstSeqCnt=9;

  %* check that the sas references table is correct;
  %* need to redirect the results because there is no override in checkDS;
  %let _cstSavedOrigResultsName=&_cstResultsDS;
  %let _cstResultsDS=&_cstThisResultsDS;

  %if (&_cstDebug) %then %do;
    %put before the call to cst_checkDS;
    %put _cstSavedOrigResultsName=&_cstSavedOrigResultsName;
    %put _cstResultsDS=&_cstResultsDS;
  %end;

  * The library for the results must exist;
  %if ((%sysfunc(libref(&_cstThisResultsDSLib)))) %then %do;
     %let _cstParam1=&_cstThisResultsDSLib;
     %goto RESULTSLIB_NOT_ASSIGNED;
  %end;

  * _cstCreateDisplayStyleSheet must be 0 or 1;
  %if (^((&_cstCreateDisplayStyleSheet=0) OR (&_cstCreateDisplayStyleSheet=1))) %then %do;
     %let _cstParam1=_cstCreateDisplayStyleSheet;
     %let _cstParam2=&_cstCreateDisplayStyleSheet;
     %goto INVALID_PARAM_VALUE;
  %end;


  %* set up static-final style variables;
  %let _cstStandard=&_cstrunstd;
  %let _cstStandardVersion=&_cstrunstdver;
  %let _cstAction=EXPORT;

  %* Get the fileref of the src data and the XML file and then the full path to it;
  %cstUtil_getSASReference(
    _cstStandard=%upcase(&_cstStandard)
    ,_cstStandardVersion=&_cstStandardVersion
    ,_cstSASRefType=&_cstTypeSourceData
    ,_cstSASRefsasref=_cstSrcDataLibrary
    );
  %if (&_cst_rc) %then %do;
     %let _cstParam1=&_cstTypeSourceData;
     %goto MISSING_SASREF;
  %end;
  %else %if (%sysfunc(libref(&_cstSrcDataLibrary))) %then %do;
     %let _cstParam1=&_cstSrcDataLibrary;
     %let _cstParam2=&_cstTypeSourceData;
     %goto MISSING_ASSIGNMENT;
  %end;
  %else %if (&_cstDebug=1) %then %do;
    %put Retrieved sourceData library from sasrefs file: &_cstSrcDataLibrary;
  %end;

  %* Get the fileref of the XML file;
  %cstUtil_getSASReference(
    _cstStandard=%upcase(&_cstStandard)
    ,_cstStandardVersion=&_cstStandardVersion
    ,_cstSASRefType=&_cstTypeExtXml
    ,_cstSASRefSubType=&_cstSubtypeXML
    ,_cstSASRefsasref=_cstTempFilename1
    );
  %if (&_cst_rc) %then %do;
     %let _cstParam1=&_cstTypeExtXml/&_cstSubtypeXML;
     %goto MISSING_SASREF;
  %end;
  %else %if (&_cstDebug=1) %then %do;
    %put XML file to create from sasrefs file: &_cstTempFilename1;
  %end;

  %* Get the fileref of the stylesheet to use file - if the user is asking to use one;
  %if (&_cstCreateDisplayStyleSheet=1) %then %do;
    %cstUtil_getSASReference(
      _cstStandard=%upcase(&_cstStandard)
      ,_cstStandardVersion=&_cstStandardVersion
      ,_cstSASRefType=&_cstTypeRefXML
      ,_cstSASRefSubType=&_cstSubtypeStylesheet
      ,_cstSASRefsasref=_cstTempFilename5
      ,_cstSASRefMember=_cstOutputStyleSheetName
      );
    %if (&_cstDebug) %then %do;
      %put _cstTempFilename5: &_cstTempFilename5;
      %put PATH: %sysfunc(pathname(&_cstTempFilename5));
    %end;
    %if (%length(&_cstTempFilename5)>0) %then %do;
      %* if provided, it must exist;
      %if ((%sysfunc(fexist(&_cstTempFilename5)))=0) %then %do;
         %let _cstParam1=%sysfunc(pathname(&_cstTempFilename5));
         %goto MISSING_FILE;
      %end;
    %end;
  %end;
  %if (&_cstDebug=1) %then %do;
    %put Stylesheet reference: &_cstTempFilename5;
    %put _cstCreateDisplayStyleSheet=&_cstCreateDisplayStyleSheet;
    %put _cstStandard=%upcase(&_cstStandard);
    %put _cstStandardVersion=&_cstStandardVersion;
    %put _cstSASRefType=&_cstTypeRefXML;
    %put _cstSASRefSubType=&_cstSubtypeStylesheet;
  %end;

  %* This next section ensures that the dataset and column names have the correct
     capitalization as SAS does not care but Java and XML do.  This will be done
     by creating a temporary lib under the work directory and creating the
     correct data sets under there.
     These will be used in the ODM XML creation and then deleted later.;

  * Get the path to the work library, create the temp dir under it and
    assign a library to it;
  data _null_;
    length
      workPath $500
      tempDirPath $500
      ;

    workPath=pathname('work');
    call symputx('_cstWorkPath',workPath,'L');
    tempDirPath=dcreate("&_cstTempDirLib1","&&_cstWorkPath");
    call symputx('_cstTempDirLib1FullPath',tempDirPath,'L');
  run;

  libname &_cstTempDirLib1 "&_cstTempDirLib1FullPath";

  * Create the standard metadata files in the new directory;
  %cst_createTablesForDataStandard(
     _cstStandard=&_cstStandard
    ,_cstStandardVersion=&_cstStandardVersion
    ,_cstOutputLibrary=&_cstTempDirLib1
    );


  * Get the library information for the table metadata for this standard-version;
  * and create a working copy if it;
  %* NOTE: this uses the same variable name for 2 purposes: the libname and the work data;
  %* set name.  Do not confuse the two things;
  data _null_;
    file &_cstNextCode;
    set &_cstGlobalMDLib..&_cstGlobalStdSASRefsDS (where=(
      (standard="&_cstStandard") AND
      (standardVersion="&_cstStandardVersion") AND
      (upcase(type)="%upcase(&_cstTypeRefMD)") AND
      (upcase(subType)="%upcase(&_cstSubTypeTable)")
    ));
    memname = scan(memname,1,'.');

    put @1 "* Assign the library to the standard's metadata;";
    put @1 'libname &_cstTempRefMdTable "' %unquote(path) +(-1) '" access=readonly;';

    put @1 "* Create the MDP data set that be used to create the XML;";
    put @1 "* This also contains the srcMemname that is the data set the user has provided;";
    put @1 "data work.&_cstTempRefMdTable(keep=srcMemname mdpMemname mdpName);";
    put @3   "length srcMemname $65 mdpMemname $65 mdpName $200;";
    put @3   "set &_cstTempRefMdTable.." memname +(-1) ";";
    put @3   "srcMemname='" "&_cstSrcDataLibrary" "' || '.' || table;";
    put @3   "mdpMemname='" "&_cstTempDirLib1" "' || '.' || table;";
    put @3   "if exist(srcMemname) then do;";
    put @5     "mdpName=xmlElementName;";
    put @5     "output;";
    put @3   "end;";
    put @1 "run;";

    put @1 "* De-assign the libname;";
    put @1 "*libname &_cstTempRefMdTable;";
  run;

  * Include the code just created to create the mdp table;
  * Note: there may be issues if this is buried in multiple macro calls;
  %include &_cstNextCode;

  data _null_;
    if 0 then set work.&_cstTempRefMdTable nobs=_numobs;
    call symputx('_cstTempRefMdTableObs',_numobs);
    stop;
  run;
   * The source data sets must exist;
   %if (&_cstTempRefMdTableObs = 0) %then %do;
      %let _cstParam1=No SAS data sets to transform to ODM in library &_cstSrcDataLibrary..;
      %let _cstParam2=;
      %goto MISSING_SOURCE_DATASETS;
   %end;

  * use the data set just created (work.&_cstTempRefMdTable) to write the
    code to copy the data sets;
  data _null_;
    file &_cstNextCode;
    set work.&_cstTempRefMdTable;

    if _n_=1 then do;
      put @1 "* This section copies the source data into the temporary library;";
    end;
    put @1 "data " mdpMemname ";";
    put @3   "if (0) then set " mdpMemname ";";
    put @3   "set " srcMemname ";";
    put @1 "run;"/ /;
  run;


  * Include the code just created to fill the correctly capitalizaed tables/columns;
  * Note: there may be issues if this is buried in multiple macro calls;
  %include &_cstNextCode;


  %* At this point the data prep is complete, so need to create the XML;

  * Assign temporary filenames to the repository paths as the SAS !Var needs to be expanded;
  %* _cstTempFilename1,5 was assigned already so leave it;
  filename &_cstTempFilename2 "%unquote(&_cstGlobalXsdPath)";
  filename &_cstTempFilename3 "%unquote(&_cstGlobalXslPath)";
  filename &_cstTempFilename4 "%unquote(&_cstGlobalMDPath)";

  * Save the path to the XML file, xsd/xsl repos and work library for use later;
  data _null_;
    * call symputx('_cstWorkPath',pathname('work'),'L');
    call symputx('_cstExternalXMLPath',pathname("&_cstTempFilename1"),'L');
    call symputx('_cstXsdReposPath',pathname("&_cstTempFilename2"),'L');
    call symputx('_cstXslReposPath',pathname("&_cstTempFilename3"),'L');
    call symputx('_cstMDPath',pathname("&_cstTempFilename4"),'L');
    %if (%length(&_cstTempFilename5)>0) %then %do;
      call symputx('_cstStyleSheetPath',pathname("&_cstTempFilename5"),'L');
    %end;
  run;


* De-Allocate the temporary filename;
  filename &_cstTempFilename2;
  filename &_cstTempFilename3;
  filename &_cstTempFilename4;

  %* the location where the temporary cube XML will be stored;
  %let _cstCubeXMLPath=&_cstWorkPath./&_cstCubeXMLName;
  %let _cstLogXMLPath=&_cstWorkPath./&_cstLogXMLName;
  %let _cstAvailableTransformsPath=&_cstMDPath./&_cstGlobalTransformsXML;

  %* Print out debuggin information;
  %if (&_cstDebug=1) %then %do;
    %put Note: Calling java with parameters:;
    %put       _cstJavaPicklist=&_cstJavaPicklist;
    %put       _cstParamsClass=&_cstParamsClass;
    %put       _cstTransformsClass=&_cstTransformsClass;
    %put       _cstAction=&_cstAction;
    %put       _cstStandard=&_cstStandard;
    %put       _cstStandardVersion=&_cstStandardVersion ;
    %put       _cstXslReposPath=&_cstXslReposPath;
    %put       _cstXsdReposPath=&_cstXsdReposPath;
    %put       _cstCubeXMLPath=&_cstCubeXMLPath;
    %put       _cstExternalXMLPath=&_cstExternalXMLPath;
    %put       _cstStyleSheetPath=&_cstStyleSheetPath;
    %put       _cstAvailableTransformsPath=&_cstAvailableTransformsPath;
    %put       _cstLogXMLPath=&_cstLogXMLPath;
    %put       _cstCreateDisplayStyleSheet=&_cstCreateDisplayStyleSheet;
    %put       _cstOutputStyleSheetName=&_cstOutputStyleSheetName;
  %end;


  %* write the intermediate XML file out;

  %*cstutil_writecubexml(_cstXMLOut=&_cstCubeXMLPath,_cstMDPFile=work.&_cstTempRefMdTable);
  %cstutil_writeODMcubexml(_cstXMLOut=&_cstCubeXMLPath, _cstEncoding=&_cstOutputEncoding);


  %* The temporary data sets are no longer needed so they can be cleaned up,
    the directory deleted and the libname deassigned;
  proc datasets nolist kill lib=&_cstTempDirLib1;
  quit;
  libname &_cstTempDirLib1;

  data _null_;
    rc=filename("&_cstTempDirLib1","&_cstTempDirLib1FullPath");
    rc=fdelete("&_cstTempDirLib1");
    rc=filename("&_cstTempDirLib1");
  run;
  %* end of cleanup;


  %* In the following, the logging level is set to info as the transform creates
     an empty XML doc if nothing is reported, which causes an error in the SAS libname;
  * Create the external XML file from intermediate xml;
  data _null_
    %* Only use picklist for SAS V9.2 and above;
    %if %eval(^ (&SYSVER=9.1)) %then %do;
         / picklist="&_cstJavaPicklist"
    %end;
    ;

    dcl javaobj prefs("&_cstParamsClass");
    prefs.callvoidmethod('setImportOrExport',"&_cstAction");
    prefs.callvoidmethod('setStandardName',"&_cstStandard");
    prefs.callvoidmethod('setStandardVersion',"&_cstStandardVersion");

    prefs.callvoidmethod('setXslBasePath',tranwrd("&_cstXslReposPath",'\','/'));
    prefs.callvoidmethod('setSchemaBasePath',tranwrd("&_cstXsdReposPath",'\','/'));

    prefs.callvoidmethod('setSasXMLPath',tranwrd("&_cstCubeXMLPath",'\','/'));
    prefs.callvoidmethod('setStandardXMLPath',tranwrd("&_cstExternalXMLPath",'\','/'));

    prefs.callvoidmethod('setAvailableTransformsFilePath',tranwrd("&_cstAvailableTransformsPath",'\','/'));
    prefs.callvoidmethod('setLogFilePath',tranwrd("&_cstLogXMLPath",'\','/'));

    if ("&_cstOutputEncoding" ne '') then do;
      prefs.callvoidmethod('setOutputEncoding',"&_cstOutputEncoding");
    end;

    if ("&_cstHeaderComment" ne '') then do;
      prefs.callvoidmethod('setHeaderCommentText',tranwrd("&_cstHeaderComment",'\','/'));
    end;


    if (&_cstCreateDisplayStyleSheet=1) then do;
      if ("&_cstStyleSheetPath" ne '') then do;
        prefs.callvoidmethod('setCustomStylesheetPath', tranwrd("&_cstStyleSheetPath",'\','/'));
        prefs.callvoidMethod('setOutputStylesheetName', tranwrd("&_cstOutputStyleSheetName..xsl",'\','/'));
      end;
      prefs.callvoidmethod('createDisplayStylesheet');
    end;

    * set logging to INFO;
    prefs.callvoidmethod('setLogLevelString',"&loglev");

    dcl javaobj transformer("&_cstTransformsClass", prefs);
    transformer.exceptiondescribe(1);
    transformer.callvoidmethod('exec');

    * check the return values here and get results path;
    transformer.delete();
    prefs.delete();
  run;


  * check to see if there are any Java issues;
  %cstutilcheckjava;
  %if (&_cstResultFlag) %then %do;
    %goto JAVA_ERRORS;
  %end;

  * check to see if the log file is empty;
  %let lfile=%unquote(&_cstLogXMLPath);
  %if %sysfunc(fileexist(&lfile)) %then %do;
    data _null_;
    length line $17;
    infile "&lfile";
    input line $;
    a=strip(line);
    found=find(a,'<XMLTransformLog>');
    if found=1 then do;
      call symput("logExists" ,"Y");
      stop;
    end;
    else call symput("logExists" ,"N");
    run;
  %end;
  %else %let logExists="N";

  %if "&logExists"="Y" %then %do;
    %* reuse the same temp variable name for file name;
    * Assign a libname to the log XML file;
    libname &_cstLogXMLName &_cstXMLEngine "%unquote(&_cstLogXMLPath)";

    * Check to see if the line number/column number info was generated;
    proc contents noprint
      data=&_cstLogXMLName..XMLTransformLog out=&_cstTempDS1;
    run;

    %let _cstLineNumberColExists=0;
    data _null_;
      set &_cstTempDS1;
      if (upcase(name)="LINENUMBER") then
        call symputx('_cstLineNumberColExists',1);
    run;

    %if (&_cstDebug) %then %do;
      %put _cstLineNumberColExists=&_cstLineNumberColExists;

      %put Results from the Java call:;
      data _null_;
        set &_cstLogXMLName..XMLTransformLog;
        put message=;
      run;
    %end;

    * Create a work results data set to capture the XML log information;
    data work._cstxmllog;
      %cstutil_resultsdskeep;
      %cstutil_resultsdsattr;
      length _cstmsgParm1 $450 _cstmsgParm2 $1;
      keep _cstmsgParm1 _cstmsgParm2;
      call missing(actual,keyvalues,resultdetails,_cstmsgParm2);

      set &_cstLogXMLName..XMLTransformLog;

      seqno=_n_;
      resultseq=1;
      checksource='XMLTRANSFORM';

      %if (&_cstLineNumberColExists) %then
      %do;
        if (lineNumber^=.) then do;
          _cstMsgParm1='(Line ' || compress(put(lineNumber,8.)) ||
             '/Column ' || compress(put(columnNumber,8.)) || ') ' || trim(left(message));
          message='(Line ' || compress(put(lineNumber,8.)) ||
             '/Column ' || compress(put(columnNumber,8.)) || ') ' || trim(left(message));
        end;
        else do;
          _cstmsgparm1=strip(message);
          message=strip(message);
        end;
      %end;
      %else
      %do;
        _cstmsgparm1=strip(message);
        message=strip(message);
      %end;

      resultseverity=severity;
      srcdata=Strip(origin);
      checkId='';
      ResultFlag=0;

      if (severity='INFO') then
        ResultId='ODM0001';
      else if (severity='WARNING') then
        ResultId='ODM0002';
      else do;
        * ERROR/CRITICAL ERROR;
        ResultId='ODM0003';
        ResultFlag=1;
        call symputx('_cstThisMacroRC','1','L');
      end;
      _cst_RC=ResultFlag;
    run;

    %cstutil_appendresultds(
       _cstErrorDS=work._cstxmllog,
       _cstSource=XMLTRANSFORM,
       _cstVersion=1.0,
       _cstOrderBy=seqno);

    * Clear the libname;
    libname &_cstLogXMLName;

    %cstutil_deleteDataSet(_cstDataSetName=work._cstxmllog);

  %end;

  * Cleanup the log and cube file;
  %if (&_cstDebug)<1 %then %do;
    data _null_;
      rc=filename("&_cstLogXMLName","&_cstLogXMLPath");
      rc=fdelete("&_cstLogXMLName");
      rc=filename("&_cstLogXMLName");
      rc=filename("&_cstCubeXMLFilename","&_cstcubeXMLPath");
      rc=fdelete("&_cstCubeXMLFilename");
      rc=filename("&_cstCubeXMLFilename");
    run;
  %end;

  %* Handle any errors generated during the java code;
  %if (&_cstThisMacroRC) %then %do;
    %goto GENERATION_ERRORS;
  %end;

  %* Everything was OK so  report it;
  %let _cstParam1=&_cstExternalXMLPath;
  %let _cstThisMacroRC=0;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0010
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );

  %goto CLEANUP;


%RESULTSLIB_NOT_ASSIGNED:
  %if (&_cstDebug) %then %do;
     %put In RESULTSLIB_NOT_ASSIGNED;
  %end;

  %local _cstTempResultsDS;

  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempResultsDS=_cst&_cstRandom;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0007
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstTempResultsDS
                );

  %* print the temp results to the log as the resutls file cannot exist;
  data _null_;
    file &_cstNextCode;
    set &_cstTempResultsDS;
    resultSeverity=upcase(resultSeverity);
    put %nrstr("%put ") resultSeverity ": "  message ";";
  run;

  %cstutil_deleteDataSet(_cstDataSetName=work.&_cstTempResultsDS);

  %include &_cstNextCode;

  %let rc=delete("&_cstNextCode");

  %goto CLEANUP;

%MISSING_DATASET:
  %if (&_cstDebug) %then %do;
     %put In MISSING_DATASET;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0008
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_SOURCE_DATASETS:
  %if (&_cstDebug) %then %do;
     %put In MISSING_SOURCE_DATASETS;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0099
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=CRTDDS_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_FILE:
  %if (&_cstDebug) %then %do;
     %put In MISSING_FILE;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0009
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_SASREF:
  %if (&_cstDebug) %then %do;
     %put In MISSING_SASREF;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0004
                ,_cstResultParm1=&_cstParam1
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%MISSING_ASSIGNMENT:
  %if (&_cstDebug) %then %do;
     %put In MISSING_ASSIGNMENT;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0005
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%INVALID_PARAM_VALUE:
  %if (&_cstDebug) %then %do;
     %put In INVALID_PARAM_VALUE;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0006
                ,_cstResultParm1=&_cstParam1
                ,_cstResultParm2=&_cstParam2
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%GENERATION_ERRORS:
  %if (&_cstDebug) %then %do;
     %put In GENERATION_ERRORS;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0011
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%JAVA_ERRORS:
  %if (&_cstDebug) %then %do;
     %put In JAVA_ERRORS;
  %end;

  %let _cstThisMacroRC=1;
  %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
  %cstutil_writeresult(
                _cstResultId=ODM0099
                ,_cstResultParm1=PROCESS HALTED
                ,_cstResultSeqParm=&_cstResultSeq
                ,_cstSeqNoParm=&_cstSeqCnt
                ,_cstSrcDataParm=ODM_WRITE
                ,_cstResultFlagParm=&_cstThisMacroRC
                ,_cstRCParm=&_cstThisMacroRC
                ,_cstResultsDSParm=&_cstThisResultsDS
                );
  %goto CLEANUP;

%CLEANUP:
  %if (&_cstDebug) %then %do;
     %put In CLEANUP;
  %end;

  %* Persist the results if specified in sasreferences  *;
  %cstutil_saveresults();

  %* reset the resultSequence/SeqCnt variables;
  %cstutil_internalManageResults(_cstAction=RESTORE);

  * Clear the libname;
  libname &_cstGlobalMDLib;

  * Clear the temporary filename into the work catalog;
  filename &_cstNextCode;

  * Clean up temporary data sets if they exist;
  %cstutil_deleteDataSet(_cstDataSetName=work.&_cstTempRefMdTable);
  %cstutil_deleteDataSet(_cstDataSetName=work.&_cstTempDS1);

  %* Delete the temporary messages data set if it was created here;
  %if (&_cstNeedToDeleteMsgs=1) %then %do;
    %local _cstMsgDir _cstMsgMem;
    %if %eval(%index(&_cstMessages,.)>0) %then %do;
      %let _cstMsgDir=%scan(&_cstMessages,1,.);
      %let _cstMsgMem=%scan(&_cstMessages,2,.);
    %end;
    %else %do;
      %let _cstMsgDir=work;
      %let _cstMsgMem=&_cstMessages;
    %end;
    %cstutil_deleteDataSet(_cstDataSetName=&_cstMsgDir..&_cstMsgMem);
  %end;

  %let _cst_rc=&_cstThisMacroRC;

%mend odm_write;
