%* sdtmutil_createformatsfromcrtdds                                               *;
%*                                                                                *;
%* Derives codelists from a CRT-DDS data library.                                 *;
%*                                                                                *;
%* This sample utility macro attempts to derive code lists from a CRT-DDS data    *;
%* library that is derived from define.xml for a CDISC SDTM study.                *;
%*                                                                                *;
%* These source metadata files are used by the SAS Clinical Standards Toolkit to  *;
%* create codelists as provided in the CDISC CRT-DDS (define.xml) files:          *;
%*          codelists                                                             *;
%*          codelistitems                                                         *;
%*          clitemdecodetranslatedtext                                            *;
%*                                                                                *;
%* This is the general strategy:                                                  *;
%*    1. Combine CRT-DDS data to create the cntlin data set.                      *;
%*    2. Read the cntlin data set using PROC FORMAT to create format catalog.     *;
%*                                                                                *;
%*                                                                                *;
%* ASSUMPTIONS:                                                                   *;
%*    1. The source data is read from a single SAS library. You can modify the    *;
%*       code to reference multiple libraries by using library concatenation.     *;
%*    2. You can specify only one study reference. Multiple study references      *;
%*       require modification of the code.                                        *;
%*    3. The following macro variables have been set either prior to calling this *;
%*       macro or that they can be derived from the _cstSASRefs data set:         *;
%*           _cstCTCat (type=fmtsearch, format catalog name)                      *;
%*           _cstCTLibrary (type=fmtsearch, target library for formats)           *;
%*           _cstSASrefLib (type=sourcedata, library of derived CDT-DDS data sets)*;
%*           _cstSrcMetaLib (type=sourcemetadata, libref for CRT-DDS source_study)*;
%*           _cstSrcStudyDS (type=sourcemetadata, source_study data set name)     *;
%*    4. The following macro variables have been set previously:                  *;
%*           _cstStandard (for example, CDISC-SDTM)                               *;
%*           _cstStandardVersion (for example, 3.1.2)                             *;
%*                                                                                *;
%* @macvar _cstSeqCnt Results: Sequence number within _cstResultSeq               *;
%* @macvar _cstSrcData Results: Source entity being evaluated                     *;
%* @macvar _cstResultsDS Results data set                                         *;
%* @macvar studyRootPath Root path of the Study Library                           *;
%* @macvar _cstCTLibrary Target library for formats                               *;
%* @macvar _cstSASrefLib Library of derived CDTDDS data sets                      *;
%* @macvar _cstSrcMetaLib Libref for CRTDDS source_study                          *;
%* @macvar _cstSrcStudyDS Source study data set name                              *;
%* @macvar _cstStandard Name of a standard registered to Toolkit                  *;
%* @macvar _cstStandardVersion Version of the standard referenced in _cstStandard *;
%* @macvar _cstSASRefs Run-time SASReferences data set derived in process setup   *;
%* @macvar _cstGRoot Root path of the Toolkit Global Library                      *;
%* @macvar _cstCTCat Format catalog name                                          *;
%*                                                                                *;
%* @since 1.3                                                                     *;
%* @exposure external                                                             *;

%macro sdtmutil_createformatsfromcrtdds(
    ) / des='CST: Create Formats from CRTDDS Codelists';

  %local
    _cstDir
    _cstErrorFlag
    _cstFileref
    _cstRandom
    _cstRC
    _cstrundt
    _cstTempDS1
    _cstVer
    _cst_Error
  ;

  %let _cstSeqCnt=0;
  %let _cstSrcData=&sysmacroname;

  %* Write information about this process to the results data set  *;
  %if %symexist(_cstResultsDS) %then
  %do;
    %cst_getRegisteredStandards(_cstOutputDS=work._cstStandards);

    data _null_;
      set work._cstStandards (where=(standard="CST-FRAMEWORK"));
      call symputx('_cstVer',strip(productrevision));
      call symputx('_cstrundt',put(datetime(),is8601dt.));
    run;
    proc datasets lib=work nolist;
      delete _cstStandards;
    quit;

    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARD: &_cstStandard,_cstSeqNoParm=1,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARDVERSION: &_cstStandardVersion,_cstSeqNoParm=2,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DRIVER: CREATE_CODELISTFROMCRTDDS,_cstSeqNoParm=3,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DATE: &_cstrundt,_cstSeqNoParm=4,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS TYPE: METADATA DERIVATION,_cstSeqNoParm=5,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS SASREFERENCES: &_cstsasrefs,_cstSeqNoParm=6,_cstSrcDataParm=&_cstSrcData);
    %if %symexist(studyRootPath) %then
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: &studyRootPath,_cstSeqNoParm=7,_cstSrcDataParm=&_cstSrcData);
    %else
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: <not used>,_cstSeqNoParm=7,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS GLOBALLIBRARY: &_cstGRoot,_cstSeqNoParm=8,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS CSTVERSION: &_cstVer,_cstSeqNoParm=9,_cstSrcDataParm=&_cstSrcData);
    %let _cstSeqCnt=9;
  %end;

  %****************************************************;
  %*  Check for existence of required LIBNAME values  *;
  %****************************************************;
  %if ^%symexist(_cstCTLibrary) %then
    %cstutil_getsasreference(_cstSASRefType=fmtsearch,_cstSASRefsasref=_cstCTLibrary,_cstSASRefmember=_cstCTCat);

  %if "&_cstCTLibrary"="" %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=ERROR Location for output data sets required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR Location for output data sets required;
  %end;

  %if ^%symexist(_cstSASrefLib) %then
    %cstutil_getsasreference(_cstSASRefType=sourcedata,_cstSASRefsasref=_cstSASrefLib);

  %if "&_cstSASrefLib"="" %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=ERROR Location for CRTDDS input data sets required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR Location for CRTDDS input data sets required;
  %end;

  %if ^%symexist(_cstSrcMetaLib) %then
    %cstutil_getsasreference(_cstSASRefType=sourcemetadata,_cstSASRefSubtype=study,_cstSASRefsasref=_cstSrcMetaLib,
                             _cstSASRefmember=_cstSrcStudyDS);

  %if "&_cstSrcMetaLib"="" %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Location for CRTDDS derived metadata source_study data set required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR Location for CRTDDS derived metadata data set required;
  %end;
  %if "&_cstSrcStudyDS"="" %then
  %do;
    %let _cstErrorFlag=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Location for CRTDDS derived metadata source_study data set required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR Location for CRTDDS derived metadata data set required;
  %end;

  %if &_cst_Error=1 %then %goto exit_macro;

  %* Look in derived CRTDDS source_study data set to determine if we are trying to rebuild the SDTM       *;
  %*  study from a define.xml file that was built for the same SDTM version.  If not, abort the process.  *;

  data _null_;
    set &_cstSrcMetaLib..&_cstSrcStudyDS;
    if upcase(standard) ne upcase("&_cstStandard") or upcase(standardversion) ne upcase("&_cstStandardVersion") then
      call symputx('_cst_Error',1);
  run;

  %let _cstSrcData=&sysmacroname;
  %if &_cst_Error=1 %then
  %do;
    %cstutil_writeresult(_cstResultID=CST0200,
                         _cstResultParm1=%str(Process halted, discrepancy noted in source and target versions of the standard),
                         _cstSeqNoParm=&_cstSeqCnt,
                         _cstSrcDataParm=&_cstSrcData);
    %goto exit_macro;
  %end;

  %*********************************************;
  %*  Check existence of required data tables  *;
  %*********************************************;
  %if not %sysfunc(exist(&_cstSASrefLib..codelists)) %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=ERROR The &_cstSASrefLib..codelists data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR: The &_cstSASrefLib..codelists data set DOES NOT EXIST!;
  %end;

  %if not %sysfunc(exist(&_cstSASrefLib..codelistitems)) %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=ERROR The &_cstSASrefLib..codelistitems data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR: The &_cstSASrefLib..codelistitems data set DOES NOT EXIST!;
  %end;

  %if not %sysfunc(exist(&_cstSASrefLib..clitemdecodetranslatedtext)) %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=ERROR The &_cstSASrefLib..clitemdecodetranslatedtext data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR: The &_cstSASrefLib..clitemdecodetranslatedtext data set DOES NOT EXIST!;
  %end;

  %let _cstDir=%sysfunc(pathname(&_cstCTLibrary));
  %let _cstRC = %sysfunc(filename(_cstFileref,&_cstDir)) ;
  %if ^%sysfunc(fexist(&_cstFileref)) %then
  %do;
    %let _cst_Error=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=ERROR The &_cstCTLibrary  catalog library DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put ERROR: Location for the &_cstCTLibrary catalog DOES NOT EXIST!;
    %end;
  %end;

  %if &_cst_Error=1 %then %goto exit_macro;

  %********************************;
  %*  Generate needed work files  *;
  %********************************;

  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS1=_cst&_cstRandom;

  proc sql noprint;
    create table work.&_cstTempDS1 as
    select ds2.codedvalue as start,
      ds2.codedvalue as end,
      ds2.rank,
      ds3.translatedtext as label,
      case when upcase(ds1.datatype)='TEXT' then 'C'
        else 'N'
      end as type,
      ds1.sasformatname as fmtname
    from &_cstSASrefLib..codelists ds1,
         &_cstSASrefLib..codelistitems ds2,
         &_cstSASrefLib..clitemdecodetranslatedtext ds3
    where (ds1.oid=ds2.fk_codelists and ds2.oid=ds3.fk_codelistitems)
    order by sasformatname, rank;
  quit;

  proc format library=&_cstCTLibrary..&_cstCTCat cntlin=work.&_cstTempDS1;
  run;

  proc format library=&_cstCTLibrary..&_cstCTCat fmtlib;
    title 'FMTLIB Output for the CTERMS Format Catalog';
  run;

  %*****************************;
  %*  Cleanup temporary files  *;
  %*****************************;
  proc datasets lib=work nolist;
    delete &_cstTempDS1/memtype=data;
  quit;

  %exit_macro:

  %let _cstSrcData=&sysmacroname;
  %if %symexist(_cstResultsDS) %then
  %do;
    %if &_cst_Error=1 %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,
                           _cstResultParm1=%str(Process failed to complete successfully, please check results data set for more information),
                           _cstSeqNoParm=&_cstSeqCnt,
                           _cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Process completed successfully,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %******************************************************;
    %* Persist the results if specified in sasreferences  *;
    %******************************************************;
    %cstutil_saveresults();
  %end;
  %else %if &_cst_Error= 1 %then
  %do;
    %let _cstSrcData=&sysmacroname;
    %put Error &_cstSrcData ended prematurely, please check log;
  %end;

%mend sdtmutil_createformatsfromcrtdds;