%* sdtmutil_createsrcmetafromcrtdds                                               *;
%*                                                                                *;
%* Derives source metadata files from a CRT-DDS data library.                     *;
%*                                                                                *;
%* This sample utility macro attempts to derive source metadata files from a      *;
%* CRT-DDS data library that is derived from the define.xml file for a CDISC SDTM *;
%* study.                                                                         *;
%*                                                                                *;
%* These source metadata files are used by the SAS Clinical Standards Toolkit to  *;
%* support CDISC SDTM  validation and to derive CDISC CRT-DDS (define.xml) files: *;
%*          source_tables                                                         *;
%*          source_columns                                                        *;
%*          source_study                                                          *;
%*                                                                                *;
%* This is the general strategy:                                                  *;
%*    1. Use PROC CONTENTS output as the primary source of the information.       *;
%*    2. Use reference_tables and reference_columns for matching the columns.     *;
%*                                                                                *;
%* NOTE:  This is ONLY an attempted approximation of source metadata. No          *;
%*        assumptions should be made that the result accurately represents the    *;
%*        study data.                                                             *;
%*                                                                                *;
%* ASSUMPTIONS:                                                                   *;
%*    1. The source data is read from a single SAS library. You can modify the    *;
%*       code to reference multiple libraries by using library concatenation.     *;
%*    2. Only one study reference can be specified. Multiple study references     *;
%*       require modification of the code.                                        *;
%*                                                                                *;
%* @macvar _cstSeqCnt Results: Sequence number within _cstResultSeq               *;
%* @macvar _cstSrcData Results: Source entity being evaluated                     *;
%* @macvar _cstResultsDS Results data set                                         *;
%* @macvar studyRootPath Root path of the Study Library                           *;
%* @macvar _cstStandard Name of a standard registered to Toolkit                  *;
%* @macvar _cstStandardVersion Version of the standard referenced in _cstStandard *;
%* @macvar _cstSASRefs Run-time SASReferences data set derived in process setup   *;
%* @macvar _cstGRoot Root path of the Toolkit Global Library                      *;
%* @macvar _cstTrgMetaLibrary Target library for CRT-DDS source data              *;
%* @macvar _cstCRTDataLib CRT-DDS source data library                             *;
%* @macvar _cstReflib CRT-DDS reference library                                   *;
%* @macvar _cstCRTMetaLibrary Library for CRT-DDS derived metadata source study   *;
%* @macvar _cstCRTStudyDS CRT-DDS derived metadata source study data set          *;
%* @macvar _cstRefTableDS CRT-DDS reference table data set                        *;
%* @macvar _cstRefColumnDS CRT-DDS reference column data set                      *;
%* @macvar _cstSDTMDataLib SDTM metadata library                                  *;
%* @macvar _cstTrgTableDS Target source column metadata data set                  *;
%* @macvar _cstTrgStudyDS Target source study metadata data set                   *;
%* @macvar _cstLRECL Logical record length setting for filename statement         *;
%*                                                                                *;
%* @since 1.3                                                                     *;
%* @exposure external                                                             *;

%macro sdtmutil_createsrcmetafromcrtdds(
    ) / des='CST: Create SDTM metadata from CRTDDS Data';

  %local
    _cstErrorFlag
    _cstNextCode
    _cstRandom
    _cstrundt
    _cstTempDS1
    _cstTempDS2
    _cstTempDS3
    _cstTempDS4
    _cstTempDS5
    _cstVer
  ;

  %let _cstSeqCnt=0;
  %let _cstSrcData=&sysmacroname;

  %* Write information about this process to the results data set  *;
  %if %symexist(_cstResultsDS) %then
  %do;
    %cst_getRegisteredStandards(_cstOutputDS=work._cstStandards);

    data _null_;
      set work._cstStandards (where=(standard="CST-FRAMEWORK"));
      call symputx('_cstVer',strip(productrevision));
      call symputx('_cstrundt',put(datetime(),is8601dt.));
    run;
    proc datasets lib=work nolist;
      delete _cstStandards;
    quit;

    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARD: &_cstStandard,_cstSeqNoParm=1,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STANDARDVERSION: &_cstStandardVersion,_cstSeqNoParm=2,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DRIVER: CREATE_SOURCEMETADATA,_cstSeqNoParm=3,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS DATE: &_cstrundt,_cstSeqNoParm=4,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS TYPE: METADATA DERIVATION,_cstSeqNoParm=5,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS SASREFERENCES: &_cstsasrefs,_cstSeqNoParm=6,_cstSrcDataParm=&_cstSrcData);
    %if %symexist(studyRootPath) %then
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: &studyRootPath,_cstSeqNoParm=7,_cstSrcDataParm=&_cstSrcData);
    %else
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS STUDYROOTPATH: <not used>,_cstSeqNoParm=7,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS GLOBALLIBRARY: &_cstGRoot,_cstSeqNoParm=8,_cstSrcDataParm=&_cstSrcData);
    %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=PROCESS CSTVERSION: &_cstVer,_cstSeqNoParm=9,_cstSrcDataParm=&_cstSrcData);
    %let _cstSeqCnt=9;
  %end;

  %let _cstErrorFlag=0;

  %*************************************************************;
  %*  Check for existence of required libraries and data sets  *;
  %*************************************************************;

  %if ^%symexist(_cstTrgMetaLibrary) %then
    %cstutil_getsasreference(_cstSASRefType=targetmetadata,_cstSASRefSubtype=table,_cstSASRefsasref=_cstTrgMetaLibrary,
                             _cstSASRefmember=_cstTrgTableDS);

  %if "&_cstTrgMetaLibrary"="" %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: Location for output data sets required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: Location for output data sets required.;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if ^%symexist(_cstCRTDataLib) %then
    %cstutil_getsasreference(_cstSASRefType=sourcedata,_cstSASRefsasref=_cstCRTDataLib);

  %if "&_cstCRTDataLib"="" %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: Location for CRTDDS input data sets required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: Location for CRTDDS input data sets required.;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if ^%symexist(_cstReflib) %then
    %cstutil_getsasreference(_cstSASRefType=referencemetadata,_cstSASRefSubtype=table,_cstSASRefsasref=_cstRefLib,
                             _cstSASRefmember=_cstRefTableDS);

  %if "&_cstReflib" ="" %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: REFERENCE libname not provided,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: REFERENCE libname not provided.;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if ^%symexist(_cstCRTMetaLibrary) %then
    %cstutil_getsasreference(_cstSASRefType=sourcemetadata,_cstSASRefSubtype=study,_cstSASRefsasref=_cstCRTMetaLibrary,
                             _cstSASRefmember=_cstCRTStudyDS);

  %if "&_cstCRTMetaLibrary"="" %then
  %do;
    %let _cstErrorFlag=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Location for CRTDDS derived metadata source_study data set required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR Location for CRTDDS derived metadata data sets required;
  %end;
  %if "&_cstCRTStudyDS"="" %then
  %do;
    %let _cstErrorFlag=1;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Location for CRTDDS derived metadata source_study data set required,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else %put ERROR Location for CRTDDS derived metadata data sets required;
  %end;

  %if &_cstErrorFlag=1 %then %goto exit_macro;

  %* Look in derived CRTDDS source_study data set to determine if we are trying to rebuild the SDTM       *;
  %*  study from a define.xml file that was built for the same SDTM version.  If not, abort the process.  *;

  data _null_;
    set &_cstCRTMetaLibrary..&_cstCRTStudyDS;
    if upcase(standard) ne upcase("&_cstStandard") or upcase(standardversion) ne upcase("&_cstStandardVersion") then
      call symputx('_cstErrorFlag',1);
  run;

  %let _cstSrcData=&sysmacroname;

  %if &_cstErrorFlag=1 %then
  %do;
    %cstutil_writeresult(_cstResultID=CST0200,
                         _cstResultParm1=%str(Process halted, discrepancy noted in source and target versions of the standard),
                         _cstSeqNoParm=&_cstSeqCnt,
                         _cstSrcDataParm=&_cstSrcData);
    %goto exit_macro;
  %end;


  %***************************************************************;
  %*  Set Error Flag to check existence of required CRTDDS Data  *;
  %***************************************************************;

  %if not %sysfunc(exist(&_cstRefLib..&_cstRefTableDS)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstRefLib..&_cstRefTableDS data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstRefLib..&_cstRefTableDS data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstRefLib..&_cstRefColumnDS)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstRefLib..&_cstRefColumnDS data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstRefLib..&_cstRefColumnDS data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..itemgroupdefs)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..itemgroupdefs data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..itemgroupdefs data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..itemgroupleaf)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..itemgroupleaf data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..itemgroupleaf data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..itemgroupleaftitles)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..itemgroupleaftitles data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..itemgroupleaftitles data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..metadataversion)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..metadataversion data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..metadataversion data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..itemdefs)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..itemdefs data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..itemdefs data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..itemgroupdefitemrefs)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..itemgroupdefitemrefs data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..itemgroupdefitemrefs data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..study)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..study data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..study data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %if not %sysfunc(exist(&_cstCRTDataLib..definedocument)) %then
  %do;
    %if %symexist(_cstResultsDS) %then
    %do;
      %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
      %cstutil_writeresult(_cstResultID=CST0200,_cstResultParm1=Warning: The &_cstCRTDataLib..definedocument data set DOES NOT EXIST!,_cstSeqNoParm=&_cstSeqCnt,_cstSrcDataParm=&_cstSrcData);
    %end;
    %else
    %do;
      %put Warning: The &_cstCRTDataLib..definedocument data set DOES NOT EXIST!;
    %end;
    %let _cstErrorFlag=1;
  %end;

  %********************************;
  %*  Error found exit the macro  *;
  %********************************;

  %if &_cstErrorFlag=1 %then %goto exit_macro;

  %***************************;
  %*  Create TABLE metadata  *;
  %***************************;

  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS1=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS2=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS3=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS4=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS5=_cst&_cstRandom;                                                                                                                                       %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstNextCode=_cst&_cstRandom;

  %************************************************************************;
  %*  Create reference_tables template data set to obtain table metadata  *;
  %*  The CRTDDS data does not contain the original reference metadata.   *;
  %************************************************************************;
  data work.&_cstTempDS1;
    set &_cstRefLib..&_cstRefTableDS;
    if _n_=0;
    stop;
  run;

  %*********************************************;
  %*  Create list of reference_tables domains  *;
  %*********************************************;
  proc contents data=work.&_cstTempDS1 out=work.&_cstTempDS2(keep=name)noprint;
  run;

  %***************************************************************;
  %*  Assign a filename for the code that will be generated      *;
  %*  Generate code to compare incoming lengths of data against  *;
  %*  the defined standard.                                      *;
  %***************************************************************;
  filename &_cstNextCode CATALOG "work.&_cstNextCode..nextcode.source" &_cstLRECL;

  data _null_;
    set work.&_cstTempDS2 end=eof;
    length line1-line5 $128 drop $255;
    retain drop;
    if _n_=1 then drop="drop";
    file &_cstNextCode;
    name=lowcase(name);
    line1="if length("||strip(left(name))||"1) gt lengthm("||strip(left(name))||") then";
    line2="do;";
    line3='put "Length of variable '||strip(left(upcase(name)))||' is too long.";';
    line4="end;";
    line5="else "||strip(left(name))||"="||strip(left(name))||"1;";
    put;
    put line1;
    put line2;
    put @2 line3;
    put line4;
    put line5;
    drop=strip(drop)||" "||strip(left(name))||"1";
    if eof then
    do;
      drop=strip(drop)||";";
      put;
      put drop;
    end;
  run;

  %***************************************************************************;
  %*  Read data from the appropriate CRTDDS tables to create as much of the  *;
  %*  reference_tables data set as possible given the data is incomplete.    *;
  %***************************************************************************;
  proc sql noprint;
    create table work.&_cstTempDS3 (drop=oid id) as
    select ds14.*, ds23.* from
      (select ds1.oid,
              upcase("&_cstSDTMDataLib") as sasref1,
              domain as table1 'Table Name',
              label as label1 'Table Label',
              class as class1 'Observation Class within Standard',
              structure as structure1 'Table Structure',
              purpose as purpose1 'Purpose',
              domainkeys as keys1 'Table Keys',
              standardname as standard1 'Name of Standard',
              standardversion as standardversion1 'Version of Standard',
              comment as comment1 'Comment'
      from &_cstCRTDataLib..itemgroupdefs ds1,
           &_cstCRTDataLib..metadataversion ds4
      where ds1.fk_metadataversion=ds4.oid) ds14
    left join
      (select ds2.id,
              href as xmlpath1 '(Relative) path to xpt file',
              title as xmltitle1 'Title for xpt file'
       from &_cstCRTDataLib..itemgroupleaf ds2,
            &_cstCRTDataLib..itemgroupleaftitles ds3
       where ds2.id=ds3.fk_itemgroupleaf) ds23
    on ds14.oid=ds23.id
    order by table1;
  quit;

  %*********************************************************************;
  %*  Add missing metadata columns with the proper lengths and labels  *;
  %*********************************************************************;
  data work.&_cstTempDS4;
    length table $32 state1 date1 $20 standardref1 $200;
    label table='Table Name'
      state1='Data Set State (Final, Draft)'
      date1='Release Date'
      standardref1='Associated reference(s) in Standard';
    set work.&_cstTempDS3;
    table=substr(left(table1),1,32);
    state1='';
    date1='';
    standardref1='';
  run;

  proc sort data=work.&_cstTempDS4;
    by table;
  run;

  %*****************************************************;
  %*  Create first pass of the Source_Tables data set  *;
  %*****************************************************;
  data &_cstTrgMetaLibrary..&_cstTrgTableDS;
    merge work.&_cstTempDS1(in=ina) work.&_cstTempDS4(in=inb);
    by table;
    if inb;
    *  Process generated code  *;
    %include &_cstNextCode;
  run;

  proc sort data=&_cstTrgMetaLibrary..&_cstTrgTableDS (drop=state standardref);
    by table;
  run;

  proc sort data=&_cstRefLib..&_cstRefTableDS out=work.&_cstTempDS5(keep=table state standardref);
    by table;
  run;

  %*****************************************************;
  %*  Create FINAL pass of the Source_Tables data set  *;
  %*  Merge against the specified reference_tables to  *;
  %*  retrieve missing column information. Set date to *;
  %*  TODAY                                            *;
  %*****************************************************;
  data &_cstTrgMetaLibrary..&_cstTrgTableDS(label="Source Table Metadata");
    merge &_cstTrgMetaLibrary..&_cstTrgTableDS(in=ina) work.&_cstTempDS5(in=inb);
    by table;
    if ina;
    date=put(today(),worddate18.);
  run;

  proc sort data=&_cstTrgMetaLibrary..&_cstTrgTableDS  out=&_cstTrgMetaLibrary..&_cstTrgTableDS(label="Source Table Metadata");
    by sasref table;
  run;

  %if %symexist(_cstResultsDS) %then
  %do;
    %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
    %cstutil_writeresult(
        _cstResultID=CST0074,
        _cstResultParm1=%sysfunc(pathname(&_cstTrgMetaLibrary)),
        _cstSeqNoParm=&_cstSeqCnt,
        _cstSrcdataParm=&_cstSrcData,
        _cstActualParm=&_cstTrgTableDS
    );
  %end;

  %*****************************;
  %*  Cleanup temporary files  *;
  %*****************************;
  * Clear the filename;
  filename &_cstNextCode;

  proc datasets lib=work nolist;
    delete &_cstTempDS1/memtype=data;
    delete &_cstTempDS2/memtype=data;
    delete &_cstTempDS3/memtype=data;
    delete &_cstTempDS4/memtype=data;
    delete &_cstNextCode/memtype=catalog;
  quit;

  %****************************;
  %*  Create COLUMN metadata  *;
  %****************************;

  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS1=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS2=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS3=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstTempDS4=_cst&_cstRandom;
  %cstutil_getRandomNumber(_cstVarname=_cstRandom);
  %let _cstNextCode=_cst&_cstRandom;

  %**************************************************************************;
  %*  Create reference_columns template data set to obtain column metadata  *;
  %*  The CRTDDS data does not contain the original reference metadata.     *;
  %**************************************************************************;
  data work.&_cstTempDS1;
    set &_cstRefLib..&_cstRefColumnDS;
    if _n_=0;
    stop;
  run;

  %**************************************************;
  %*  Create list of columns from reference_column  *;
  %**************************************************;
  proc contents data=work.&_cstTempDS1 out=work.&_cstTempDS2(keep=name)noprint;
  run;

  %***************************************************************;
  %*  Assign a filename for the code that will be generated      *;
  %*  Generate code to compare incoming lengths of data against  *;
  %*  the defined standard.                                      *;
  %***************************************************************;
  filename &_cstNextCode CATALOG "work.&_cstNextCode..nextcode.source" &_cstLRECL;

  data _null_;
    set work.&_cstTempDS2 end=eof;
    length line1-line5 $128 drop $255;
    retain drop;
    if _n_=1 then drop="drop";
    file &_cstNextCode;
    name=lowcase(name);
    line1="if length("||strip(left(name))||"1) gt lengthm("||strip(left(name))||") then";
    line2="do;";
    line3='put "Length of variable '||strip(left(upcase(name)))||' is too long.";';
    line4="end;";
    line5="else "||strip(left(name))||"="||strip(left(name))||"1;";
    put;
    put line1;
    put line2;
    put @2 line3;
    put line4;
    put line5;
    drop=strip(drop)||" "||strip(left(name))||"1";
    if eof then
    do;
      drop=strip(drop)||";";
      put;
      put drop;
    end;
  run;

  %***************************************************************************;
  %*  Read data from the appropriate CRTDDS tables to create as much of the  *;
  %*  reference_columns data set as possible given the data is incomplete.   *;
  %***************************************************************************;
  proc sql noprint;
    create table work.&_cstTempDS3 as
    select upcase("&_cstSDTMDataLib") as sasref1,
      domain as table1 'Table Name',
      ds1.name as column1 'Column Name',
      ds1.label as label1 'Column Description',
      ordernumber as order1 'Column Order',
      case when upcase(datatype)='TEXT' then 'C'
        else 'N'
      end as type1 'Column Type',
      length as length1 'Column Length',
      displayformat as displayformat1 'Display Format',
      datatype as xmldatatype1 'XML Data Type',
      codelistref as xmlcodelist1 'SAS Format/XML Codelist',
      ds1.origin as origin1 'Column Origin',
      ds2.role as role1 'Column Role',
      standardname as standard1 'Name of Standard',
      standardversion as standardversion1 'Version of Standard',
      ds1.comment as comment1 'Comment',
      ds5.method as algorithm1 'Computational Algorithm or Method'
    from &_cstCRTDataLib..itemdefs ds1,
      &_cstCRTDataLib..itemgroupdefitemrefs ds2,
      &_cstCRTDataLib..metadataversion ds3,
      &_cstCRTDataLib..itemgroupdefs ds4,
      (select method, dsid.oid
         from &_cstCRTDataLib..itemdefs dsid left join &_cstCRTDataLib..computationmethods dscm
         on dsid.computationmethodoid=dscm.oid) as ds5
    where (ds1.oid=ds2.itemoid and ds1.fk_metadataversion=ds3.oid and ds2.fk_itemgroupdefs=ds4.oid and ds5.oid=ds2.itemoid)
    order by domain;
  quit;


  %*********************************************************************;
  %*  Add missing metadata columns with the proper lengths and labels  *;
  %*********************************************************************;
  data work.&_cstTempDS4;
    length table $32 standardref1 qualifiers1 $200 core1 $10 term1 $80;
    label table='Table Name'
      core1='Column Required or Optional'
      term1='Controlled Term or Format in Standard'
      qualifiers1='Column qualifiers (space delimited)'
      standardref1='Associated reference(s) in Standard';
    set work.&_cstTempDS3;
    table=substr(left(table1),1,32);
    core1='';
    term1='';
    standardref1='';
    qualifiers1='';
  run;

  proc sort data=work.&_cstTempDS4;
    by table;
  run;

  %******************************************************;
  %*  Create first pass of the Source_Columns data set  *;
  %******************************************************;
  data &_cstTrgMetaLibrary..&_cstTrgColumnDS;
    merge work.&_cstTempDS1(in=ina) work.&_cstTempDS4(in=inb);
    by table;
    if inb;
    *  Process generated code  *;
    %include &_cstNextCode;
  run;

  proc sort data=&_cstTrgMetaLibrary..&_cstTrgColumnDS (drop=core term qualifiers standardref);
    by table column;
  run;

  proc sort data=&_cstRefLib..&_cstRefColumnDS out=work.&_cstTempDS5(keep=table column core term qualifiers standardref);
    by table column;
  run;

  %******************************************************;
  %*  Create FINAL pass of the Source_Columns data set  *;
  %*  Merge against the specified reference_tables to   *;
  %*  retrieve missing column information.              *;
  %******************************************************;
  data &_cstTrgMetaLibrary..&_cstTrgColumnDS(label="Source Column Metadata");
    merge &_cstTrgMetaLibrary..&_cstTrgColumnDS(in=ina) work.&_cstTempDS5(in=inb);
    by table column;
    if ina;
  run;

  proc sort data=&_cstTrgMetaLibrary..&_cstTrgColumnDS out=&_cstTrgMetaLibrary..&_cstTrgColumnDS(label="Source Column Metadata");
    by sasref table order;
  run;

  %if %symexist(_cstResultsDS) %then
  %do;
    %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
    %cstutil_writeresult(
        _cstResultID=CST0074,
        _cstResultParm1=%sysfunc(pathname(&_cstTrgMetaLibrary)),
        _cstSeqNoParm=&_cstSeqCnt,
        _cstSrcdataParm=&_cstSrcData,
        _cstActualParm=&_cstTrgColumnDS
    );
  %end;

  %*****************************;
  %*  Cleanup temporary files  *;
  %*****************************;
  * Clear the filename;
  filename &_cstNextCode;

  proc datasets lib=work nolist;
    delete &_cstTempDS1/memtype=data;
    delete &_cstTempDS2/memtype=data;
    delete &_cstTempDS3/memtype=data;
    delete &_cstTempDS4/memtype=data;
    delete &_cstTempDS5/memtype=data;
    delete &_cstNextCode/memtype=catalog;
  quit;

  %****************************;
  %*  Create STUDY metadata  *;
  %****************************;
  proc sql noprint;
    create table &_cstTrgMetaLibrary..&_cstTrgStudyDS as
    select upcase("&_cstSDTMDataLib") as sasref 'SAS metadata libref',
      ds2.id as definedocumentname 'Unique study ID',
      ds1.studyname 'Study name',
      ds1.studydescription 'Description of the study',
      ds1.protocolname 'Protocol name'
    from &_cstCRTDataLib..study ds1,
      &_cstCRTDataLib..definedocument ds2
    where (ds1.fk_definedocument=ds2.fileoid);
  quit;

  proc sort data=&_cstTrgMetaLibrary..&_cstTrgStudyDS out=&_cstTrgMetaLibrary..&_cstTrgStudyDS (label="Source Study Metadata");
    by definedocumentname studyname sasref;
  run;

  %if %symexist(_cstResultsDS) %then
  %do;
    %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
    %cstutil_writeresult(
        _cstResultID=CST0074,
        _cstResultParm1=%sysfunc(pathname(&_cstTrgMetaLibrary)),
        _cstSeqNoParm=&_cstSeqCnt,
        _cstSrcdataParm=&_cstSrcData,
        _cstActualParm=&_cstTrgStudyDS
    );
  %end;

  %exit_macro:

    %let _cstSrcData=&sysmacroname;
    %if %symexist(_cstResultsDS) %then
    %do;
      %if &_cstErrorFlag=1 %then
      %do;
        %let _cstSeqCnt=%eval(&_cstSeqCnt+1);
        %cstutil_writeresult(_cstResultID=CST0200,
                             _cstResultParm1=%str(Process failed to complete successfully, please check results data set for more information),
                             _cstSeqNoParm=&_cstSeqCnt,
                             _cstSrcDataParm=&_cstSrcData);
      %end;

      %******************************************************;
      %* Persist the results if specified in sasreferences  *;
      %******************************************************;
      %cstutil_saveresults();
    %end;

%mend sdtmutil_createsrcmetafromcrtdds;