%* sdtmutil_getchecks                                                             *;
%*                                                                                *;
%* Retrieves SDTM Validation checks.                                              *;
%*                                                                                *;
%* This macro specifically serves users who run validation checks in SAS Clinical *;
%* Data Integration.                                                              *;
%*                                                                                *;
%*                                                                                *;
%* @macvar  _cstMacroVariable description                                         *;
%*                                                                                *;
%* @param   _cstParameter description                                             *;
%*                                                                                *;
%* @since 1.2                                                                     *;
%* @exposure external                                                             *;

%macro sdtmutil_getchecks (
    _cstControl=,
    _cstMeta=,
    _cstMsg=,
    _cstDomain="*",
    _cstOutDS=,
    _cstIncludeDraft=true
    ) / des='CST: Retrieve SDTM Validation Checks';

/* you can only pass in one domain at a time*/
%local rnd;

%if "&_cstIncludeDraft"="true"
  %then  %let whrdraft=;
%else
  %let whrdraft=where checkstatus=1;

%if &_cstDomain ne "*" %then %do;
%cstutil_getRandomNumber(_cstVarname=rnd);
 %let wtable=_c&rnd;
 %let wtable2=_c2&rnd;

%let numDomains=%sysfunc(sum(%sysfunc(countc(&_cstdomain,',')),1));

data _null_;
if findc(&_cstdomain,'"') then
  dom=compress(&_cstdomain,'"');
else if findc(&_cstdomain,"'") then ;
  dom=compress(&_cstdomain,"'");
call symput('_cstdoms',dom);
run;

%do i=1 %to &numDomains;
 data _null_;
   dom=scan("&_cstdoms",&i,',');
   call symput('dom',dom);
 run;
proc sql noprint;
  create table &wtable(drop=resultid) as
   select *
      from &_cstControl a left join  &_cstMsg(keep=sourcedescription messagetext resultid) b
           on a.checkid = b.resultid
       &whrdraft ;
      quit;
proc sql noprint;
   select upcase(class) into :_cstClass from &_cstMeta (where=(table="&dom"));
   create table &wtable2 as
   select * from &wtable
    where ((tablescope = "ALL" or tablescope ? strip("&_cstClass") or
         indexw(tablescope,"&dom ","+") or indexw(tablescope,"&dom "," ") or
         index(tablescope,"+&dom ")));
quit;
%if &i=1 %then %do;
  data &_cstOutDS; set &wtable2; run;
%end;
%else %do;
proc append base=&_cstOutDS data=&wtable2; run;
%end;
proc sort data=&_cstOutDS nodupkey; by uniqueid; run;
proc datasets nodetails nolist;
delete &wtable &wtable2;
quit;
%end; /*do loop*/
%end;
%else %do;
proc sql noprint;
  create table &_cstOutDS(drop=resultid) as
   select *
      from &_cstControl a left join  &_cstMsg(keep=sourcedescription messagetext resultid) b
        on a.checkid = b.resultid
      &whrdraft ;
      quit;
proc sort data=&_cstOutDS nodupkey; by uniqueid; run;
%end;
%mend;
