%* sdtmutil_listsettings                                                          *;
%*                                                                                *;
%* Creates a list of settings (such as macro variables, libnames, and options).   *;
%*                                                                                *;
%* @macvar SASSERVERNAME Contains an identifier for the operating environment     *;
%* @macvar SYSSCPL Contains an identifier for the operating environment           *;
%*                                                                                *;
%* @param  group - required - The settings group to list.                         *;
%*             Values: ALL|LIBS|OPTIONS|MVARS                                     *;
%*             Default: ALL                                                       *;
%*                                                                                *;
%* @since 1.2                                                                     *;
%* @exposure external                                                             *;

%macro sdtmutil_listsettings(
    group=ALL
    ) / des="CST: List settings.";

  %local upcaseGroup;

  %let upcaseGroup=%upcase(&group.);

  %if &upcaseGroup. ^= ALL and &upcaseGroup. ^= MVARS and &upcaseGroup. ^= LIBS and  &upcaseGroup. ^= OPTIONS %then
  %do;
    %let upcaseGroup=ALL;
  %end;

  TITLE;
  FOOTNOTE;
  FOOTNOTE1 "Generated by the SAS System (&_SASSERVERNAME, &SYSSCPL) on %SYSFUNC(DATE(), EURDFDE9.) at %SYSFUNC(TIME(), TIMEAMPM8.)";

  %if &upcaseGroup.=ALL or &upcaseGroup.=LIBS %then
  %do;
    TITLE;
    TITLE1 "CST LIBREFs";
    PROC SQL;
       SELECT VLIBNAM.libname,
         VLIBNAM.level,
         VLIBNAM.path
       FROM SASHELP.VLIBNAM AS VLIBNAM
       WHERE VLIBNAM.libname NOT IN ("GISMAPS", "MAPS", "SASHELP", "SASUSER", "WORK")
         AND PRXMATCH( PRXPARSE('/^EC\d{6,6}$/') , VLIBNAM.libname) = 0
       ORDER BY VLIBNAM.libname, VLIBNAM.level, VLIBNAM.path
     ;
    QUIT;
    TITLE;
  %end;

  %if &upcaseGroup.=ALL or &upcaseGroup.=MVARS %then
  %do;
    TITLE;
    TITLE1 "CST Macro Variables";
    PROC SQL;
     SELECT name,
            ((offset / 200) + 1) label="Long Values Are Stored in Chunks" AS ChunkNumber,
            value
     FROM SASHELP.VMACRO
     WHERE lowcase(name) LIKE "_cst%" or scope="LOCAL"
     ORDER BY name, ChunkNumber;
    QUIT;
    TITLE;
  %end;

  %if &upcaseGroup.=ALL or &upcaseGroup.=OPTIONS %then
  %do;
    TITLE;
    TITLE1 "CST SAS Options";
    PROC SQL;
     SELECT VOPTION.group,
            VOPTION.optname,
            VOPTION.setting
     FROM SASHELP.VOPTION AS VOPTION
     WHERE upcase(VOPTION.group) NOT IN ('ENVDISPLAY', 'EMAIL', 'DATAQUALITY', 'UNKNOWN', 'SORT', 'INPUTCONTROL', 'HELP', 'GRAPHICS', 'COMMUNICATIONS')
     ORDER BY VOPTION.group;
    QUIT;
    TITLE;
  %end;

  TITLE; FOOTNOTE;

%mend sdtmutil_listsettings;
